import os
import sys
import time
import re
import psutil
import tensorflow as tf

from tqdm.auto import tqdm
from threading import Thread, Lock
from tensorflow.python.distribute.cluster_resolver import tpu_cluster_resolver as resolver

from tpubar import env
from tpubar.host import queryhw
from tpubar.utils import FormatSize
from tpubar.network import TimeSeriesMonitor, get_workers_list, tpunicorn_query


if env['profiler']:
    from tensorflow.python.profiler import profiler_client
    from tensorflow.python.profiler import profiler_v2 as profiler


_mesh_memory = {
    'v2-8': 6.872e+10,
    'v2-32': 2.749e+11,
    'v2-128': 1e+12,
    'v2-256': 2e+12,
    'v2-512': 4e+12,
    'v3-8': 1.374e+11,
    'v3-32': 5.498e+11,
    'v3-64': 1e+12,
    'v3-128': 2e+12,
    'v3-256': 4e+12,
    'v3-512': 8e+12
}


_timer_formats = {
    'secs': ['sec', 'secs', 'second', 'seconds', 's'],
    'mins': ['min', 'mins', 'minute', 'minutes', 'm'],
    'hrs': ['hr', 'hrs', 'hour', 'hours', 'h'],
    'days': ['day', 'days', 'd'],
    'wks': ['wk', 'wks', 'week', 'weeks' 'w']
}



class TPUMonitor:
    def __init__(self, tpu_name=None, project=None, profiler='v1', refresh_secs=10, fileout=None, verbose=False, disable=False, tpu_util='green', tpu_secondary='yellow', cpu_util='blue', ram_util='blue'):
        if profiler == 'trace':
            self.tpu_init_tf2(tpu_name)
        elif profiler in ['v1', 'v2']:
            self.tpu_init_tf2(tpu_name) if profiler == 'v2' else self.tpu_init_tf1(tpu_name, project)
        elif env['profiler'] or env['colab']:
            self.tpu_init_tf2(tpu_name)
        else:
            self.tpu_init_tf1(tpu_name, project)

        self.refresh_secs = refresh_secs
        self.fileout = fileout if fileout else sys.stdout
        self.verbose = verbose
        self.bars_disabled = disable
        self.colors = {
            'tpu_util': tpu_util,
            'tpu_secondary': tpu_secondary,
            'cpu_util': cpu_util,
            'ram_util': ram_util
        }
        self.current_stats = {}
        self.hooks = {}
        self.idx = 0
        self.hwdata()
        self.time = time.time()
        self._lock = Lock()

    def start(self, daemon=True):
        _tpubarformat = f'TPU {self.mesh} Matrix Units: ' + '{bar} {percentage:.02f}% Utilization'
        if self.profiler_ver == 'v2':
            _tpusecondarybarformat = f'TPU {self.mesh} Active Time: ' + '{bar} {percentage:.02f}% Utilization'  
        else:
            _tpusecondarybarformat = f'TPU {self.mesh} Memory: ' + '{desc} {bar} {percentage:.02f}% Utilization'  
        _cpubarformat = f'CPU {self.cpu}: ' + '{bar} {percentage:.02f}% Utilization'
        _rambarformat = 'RAM {desc} {bar} {percentage:.02f}% Utilization'
        self.tbar = tqdm(range(100), colour=self.colors['tpu_util'], bar_format=_tpubarformat, position=0, dynamic_ncols=True, leave=True, file=self.fileout, disable=self.bars_disabled)
        self.t2bar = tqdm(range(100), colour=self.colors['tpu_secondary'], bar_format=_tpusecondarybarformat, position=1, dynamic_ncols=True, leave=True, file=self.fileout, disable=self.bars_disabled)
        self.cbar = tqdm(range(100), colour=self.colors['cpu_util'], bar_format=_cpubarformat, position=2, dynamic_ncols=True, leave=True, file=self.fileout, disable=self.bars_disabled)
        self.rbar = tqdm(range(100), colour=self.colors['ram_util'], bar_format=_rambarformat, position=3, dynamic_ncols=True, leave=True, file=self.fileout, disable=self.bars_disabled)
        if daemon:
            self.alive = True
            _background = Thread(target=self.background, daemon=True)
            _background.start()
    
    def update(self):
        self.idx += 1
        tpu_stats = self.tpu_profiler()
        if tpu_stats['tpu_mxu']:
            self.tbar.n = tpu_stats['tpu_mxu']
            
        if self.profiler_ver == 'v2':
            idle_time = tpu_stats.get('tpu_idle_time', None)
            if idle_time:
                self.t2bar.n = (100.00 - idle_time)

        else:
            tpu_mem = tpu_stats.get('tpu_mem_per', None)
            if tpu_mem:
                self.t2bar.n = tpu_mem
                self.t2bar.set_description(tpu_stats.get('tpu_mem_str', ''), refresh=False)
        
        cpu_util = self.cpu_utilization()
        self.cbar.n = cpu_util
        rperc, rutil, rutilstr = self.ram_utilization()
        tpu_stats.update({'cpu_util': cpu_util, 'ram_per': rperc, 'ram_util': rutil, 'ram_util_str': rutilstr})
        self.rbar.n = rperc
        self.rbar.set_description(rutilstr, refresh=False)
        self.current_stats = tpu_stats
        self.refresh_all()
        self.fire_hooks(str(tpu_stats))

    def refresh_all(self):
        if (self.idx+1) % 10 == 0:
            self.clearbars()
        self.tbar.refresh()
        self.t2bar.refresh()
        self.cbar.refresh()
        self.rbar.refresh()

    def log(self, message):
        if not isinstance(message, str):
            message = str(message)
        message = message + '\n' + ('------' * 25)
        self.tbar.write(message)

    def background(self):
        while self.alive:
            with self._lock:
                try:
                    self.update()
                    time.sleep(self.refresh_secs)
                
                except KeyboardInterrupt:
                    self.log('Exiting')
                    self.alive = False
                    self.closebars()
                
                except Exception as e:
                    if self.verbose:
                        self.log(f'Another TPU Profiler is Active. Pausing. Error: {str(e)}')
                    time.sleep(60)
                    pass
                if not self.alive:
                    break

    def hwdata(self):
        cpu_data = queryhw()
        self.cpu = cpu_data['name'].replace('CPU', '').strip() + ' ' + str(cpu_data['cores']) + 'vCPU/' + str(cpu_data['threads']) + ' Threads'

    def tpu_util(self):
        stats = {'tpu_idle_time': 100.00, 'tpu_idle_str': '', 'tpu_mxu': 0.00, 'tpu_mxu_str': ''}
        util = self.tpu_utilization(self.service_addr, self.duration_ms, self.monitoring_level)
        util = util.split('\n')
        for stat in util:
            if 'TPU idle time' in stat:
                stats['tpu_idle_time'] = float(stat.split(':')[-1].replace('%','').strip())
                stats['tpu_idle_str'] = stat.split('  ')[-1].strip()
            elif 'Utilization of TPU' in stat:
                stats['tpu_mxu'] = float(stat.split(':')[-1].replace('%','').strip())
                stats['tpu_mxu_str'] = ' ' + stat.strip()
        return stats
    
    def tpu_api(self):
        mxu = self.monitor('tpu_core_mxu')
        for x, lst in mxu.items():
            curr_mxu = lst[0][-1]
        mem = self.monitor('tpu_container_mem')
        for x, lst in mem.items():
            curr_mem = lst[0][-1]
        mem_used, mem_str = FormatSize(curr_mem)
        vm_cpu = self.monitor('vm_cpu')
        for x, lst in vm_cpu.items():
            curr_cpu = lst[0][-1]
        tpu_cpu = self.monitor('tpu_host_cpu')
        for x, lst in tpu_cpu.items():
            curr_tpucpu = lst[0][-1]
        if self.tpu_max_mem <= curr_mem:
            self.tpu_max_mem = curr_mem + 1e+9
        mem_perc = curr_mem / self.tpu_max_mem
        _, total_mem_str = FormatSize(self.tpu_max_mem)
        
        stats = {
            'tpu_mxu': curr_mxu,
            'tpu_mem_per': (mem_perc * 100),
            'tpu_mem_util': mem_used,
            'tpu_mem_str': f'{mem_str}/{total_mem_str}',
            'tpu_vm_cpu_per': curr_cpu,
            'tpu_host_cpu_per': curr_tpucpu,
        }
        return stats

    def tpu_init_tf1(self, tpu_name=None, project=None):
        if tpu_name:
            os.environ['TPU_NAME'] = tpu_name
        tpu_config = tpunicorn_query(project)
        self.monitor = TimeSeriesMonitor(project_id=project)
        self.mesh = tpu_config['mesh']
        self.tpu_max_mem = _mesh_memory[self.mesh]
        self.profiler_ver = 'v1'
        self.tpu_profiler = self.tpu_api

    def tpu_init_tf2(self, tpu_name=None):
        tpu_name = tpu_name if tpu_name else os.environ.get('TPU_NAME', None)
        tpu_cluster_resolver = resolver.TPUClusterResolver(tpu_name)
        service_addr = tpu_cluster_resolver.get_master()
        self.service_addr = service_addr.replace('grpc://', '').replace(':8470', ':8466')
        self.workers_list = get_workers_list(tpu_cluster_resolver)
        self.monitoring_level = 2
        self.duration_ms = 1000
        util = self.tpu_utilization(self.service_addr, self.duration_ms, self.monitoring_level)
        util = util.split('\n')
        mesh_type = {'v': 'v2', 'cores': 8}
        for stat in util:
            if 'TPU type' in stat:
                tpu = stat.replace('TPU type: TPU', '').strip()
                mesh_type['v'] = tpu
            elif 'Number of TPU cores' in stat:
                idx = stat.find('(')
                mesh_cores = stat[:idx].strip()
                mesh_type['cores'] = re.search(r'[0-9]', mesh_cores).group()
        
        self.mesh = f'{mesh_type["v"]}-{mesh_type["cores"]}'
        self.tpu_max_mem = _mesh_memory[self.mesh]
        self.profiler_ver = 'v2'
        self.tpu_profiler = self.tpu_util

    def get_time(self, fmt='mins'):
        _stoptime = time.time()
        total_time = _stoptime - self.time
        if fmt in _timer_formats['wks']:
            total_time /= 604800
        elif fmt in _timer_formats['days']:
            total_time /= 86400
        elif fmt in _timer_formats['hrs']:
            total_time /= 3600
        elif fmt in _timer_formats['mins']:
            total_time /= 60
        return total_time

    def add_hook(self, name, hook, freq=10):
        self.hooks[name] = {'freq': freq, 'function': hook}
        self.log(f'Added new hook {name}. Will call hook once every {freq} updates.')

    def rm_hook(self, name):
        if self.hooks.get(name, None):
            hook = self.hooks.pop(name)
            self.log(f'Removing hook {name}')
        else:
            self.log(f'Hook {name} not found')

    def fire_hooks(self, message, *args, **kwargs):
        if self.verbose:
            self.log(message)
        if self.hooks:
            for hook_name in self.hooks:
                hook = self.hooks[hook_name]
                if self.idx % hook['freq'] == 0:
                    hook['func'](message, *args, **kwargs)


    @classmethod
    def tpu_utilization(cls, service_addr, duration_ms, monitoring_level):
        return profiler_client.monitor(service_addr, duration_ms, monitoring_level)
    
    @classmethod
    def cpu_utilization(cls):
        return psutil.cpu_percent()
    
    @classmethod
    def ram_utilization(cls):
        ram = psutil.virtual_memory()
        rused, rusedstr = FormatSize(ram.used)
        _, rtotal = FormatSize(ram.total)
        rutil = f'{rusedstr}/{rtotal}'
        return ram.percent, rused, rutil
    
    def trace(self):
        self.trace_dir = os.path.join(env['dir'], 'logs')
        os.makedirs(self.trace_dir, exist_ok=True)
        options = profiler.ProfilerOptions(host_tracer_level=self.monitoring_level)
        while self.alive:
            with self._lock:
                try:
                    profiler_client.trace(self.service_addr, self.trace_dir, self.duration_ms, self.workers_list, 5, options)
                except KeyboardInterrupt:
                    self.alive = False
                    print('Closing Tracer')
                    sys.exit()
    
    def clearbars(self):
        self.tbar.clear()
        self.t2bar.clear()
        self.cbar.clear()
        self.rbar.clear()
    
    def close(self, *_):
        self.closebars()

    def closebars(self):
        self.alive = False
        self.tbar.close()
        self.t2bar.close()
        self.cbar.close()
        self.rbar.close()

    def __exit__(self, *_):
        self.closebars()
    
    def __enter__(self):
        return self


import os
import json
env = dict()

try:
    import google.colab
    env['colab'] = True
except ImportError:
    env['colab'] = False

os.environ['TF_CPP_MIN_LOG_LEVEL'] = '0'
import tensorflow as tf
env['tf2'] = True if tf.__version__.startswith('2') else False
try:
    from tensorflow.python.profiler import profiler_client
    from tensorflow.python.framework import errors
    env['profiler'] = True
except ImportError:
    env['profiler'] = False

env['dir'] = os.path.abspath(os.path.dirname(__file__))
env['auth_path'] = os.path.join(env['dir'], 'auth.json')
auths = json.load(open(env['auth_path']))

def update_auth(updated_auths):
    json.dump(updated_auths, open(env['auth_path'], 'w'), indent=1)

if auths.get('DEFAULT_ADC', None):
    os.environ['GOOGLE_APPLICATION_CREDENTIALS'] = auths['DEFAULT_ADC']

elif os.environ.get('GOOGLE_APPLICATION_CREDENTIALS', None):
    if not auths.get('DEFAULT_ADC', None):
        auths['DEFAULT_ADC'] = os.environ['GOOGLE_APPLICATION_CREDENTIALS']
        update_auth(auths)
else:
    if env['colab']:
        print('Authenticating with Google Cloud Engine to access TPUs')
        from google.colab import auth
        auth.authenticate_user()
        auths['DEFAULT_ADC'] = os.environ.get('GOOGLE_APPLICATION_CREDENTIALS', '/content/adc.json')
        update_auth(auths)

    else:
        print('No GOOGLE_APPLICATION_CREDENTIALS Detected as Environment Variable. Run "tpubar auth auth_name" to set your ADC.')

def set_auth(auth_name):
    if auth_name in auths.keys():
        print(f'Setting ADC to {auth_name}: {auths[auth_name]}')
        if auths[auth_name] in auths.values():
            auths['BACKUP_ADC_PATH'] = auths[auth_name]
        auths['DEFAULT_ADC'] = auths[auth_name]
        os.environ['GOOGLE_APPLICATION_CREDENTIALS'] = auths[auth_name]
        update_auth(auths)
    else:
        print(f'Not able to find {auth_name} in Auth File. Update it first using "tpu auth {auth_name}".')


import tpubar.utils
import tpubar.host
import tpubar.network
import tpubar.monitor
from tpubar.monitor import TPUMonitor


from subprocess import check_output

def FormatSize(bytes, suffix="B"):
    factor = 1024
    for unit in ["", "K", "M", "G", "T", "P"]:
        if bytes < factor:
            return bytes, f"{bytes:.2f}{unit}{suffix}"
        bytes /= factor

def run_command(cmd):
    out = check_output(cmd, shell=True)
    if isinstance(out, bytes):
        out = out.decode('utf8')
    return out


import psutil
import platform

from tpubar.utils import run_command

def queryhw():
    host_os = platform.system()
    if host_os == 'Linux':
        cpu_name = run_command("lscpu |grep 'Model name'")
        cpu_name = cpu_name.split(':')[-1].strip()
    elif host_os == 'Darwin':
        # dunno why a TPU would run on macos but i kept it anyways
        cpu_name = run_command("sysctl -n machdep.cpu.brand_string | sed -e 's/ *$//'").strip()
    else:
        cpu_name = platform.processor()

    cores = psutil.cpu_count(logical=False)
    threads = psutil.cpu_count(logical=True)
    return {'name': cpu_name, 'cores': cores, 'threads': threads}



import os
import sys
import re
import calendar
import collections
import simdjson as json
import time

import google.auth

from datetime import datetime
from google.cloud import monitoring_v3
from google.protobuf.json_format import MessageToJson
from tpubar import env

if env['profiler']:
    from tensorflow.python.framework import errors

parser = json.Parser()

def flatten(d, parent_key='', sep='_'):
    items = []
    for k, v in d.items():
        new_key = parent_key + sep + k if parent_key else k
        if isinstance(v, collections.MutableMapping):
            items.extend(flatten(v, new_key, sep=sep).items())
        else:
            items.append((new_key, v))
    return dict(items)


def pb_to_json(pb):
    """Converts arbitrary protobuf messages into JSON"""
    return MessageToJson(pb)


def pb_to_dict(pb):
    """Converts arbitrary protobuf messages into python dicts"""
    return parser.parse(pb_to_json(pb)).as_dict()

def utc():
    d = datetime.utcnow()
    return calendar.timegm(d.utctimetuple())



metrics = {
    'vm_cpu': "compute.googleapis.com/instance/cpu/utilization",
    'vm_net_sent': "compute.googleapis.com/instance/network/sent_bytes_count",
    'vm_net_recv': "compute.googleapis.com/instance/network/received_bytes_count",
    'vm_disk_write': "compute.googleapis.com/instance/disk/write_bytes_count",
    'vm_disk_read': "compute.googleapis.com/instance/disk/read_bytes_count",
    'tpu_core_mxu': "tpu.googleapis.com/tpu/mxu/utilization",
    'tpu_container_cpu': "tpu.googleapis.com/container/cpu/utilization",
    'tpu_container_mem': "tpu.googleapis.com/container/memory/usage",
    'tpu_host_cpu': "tpu.googleapis.com/cpu/utilization",
    'tpu_host_mem': "tpu.googleapis.com/memory/usage",
    'tpu_host_net_sent': "tpu.googleapis.com/network/sent_bytes_count",
    'tpu_host_net_recv': "tpu.googleapis.com/network/received_bytes_count",
}

def gce_series_info(series):
    h = {k: pb_to_dict(getattr(series, k)) for k in "metric resource metadata".split()}
    h = {k: v for k, v in h.items() if len(v) > 0}
    return flatten(h)


def gce_instance_labeler(series, **options):
    if options.get('short'):
        return series.metric.labels['instance_name']
    r = []
    r += [k+'/'+series.resource.labels[k] for k in 'project_id zone'.split()]
    r += [k+'/'+series.metric.labels[k] for k in 'instance_name'.split()]
    return '/'.join(r)


def gce_instance_disk_labeler(series, **options):
    if options.get('short'):
        return '/'.join([series.metric.labels[k] for k in 'instance_name device_name'.split()])
    r = []
    r += [k+'/'+series.resource.labels[k] for k in 'project_id zone'.split()]
    r += [k+'/'+series.metric.labels[k] for k in 'instance_name device_name'.split()]
    return '/'.join(r)


def gce_series_getattrs(series, attrs, *, short=False):
    if isinstance(attrs, str):
        attrs = attrs.split()
    if short:
        r  = [series.resource.labels[k] for k in attrs if len(series.resource.labels[k]) > 0]
        r += [series.metric.labels[k] for k in attrs if len(series.metric.labels[k]) > 0]
    else:
        r  = [k+'/'+series.resource.labels[k] for k in attrs if len(series.resource.labels[k]) > 0]
        r += [k+'/'+series.metric.labels[k] for k in attrs if len(series.metric.labels[k]) > 0]
    return '/'.join(r)


def gce_tpu_labeler(series, **options):
    if options.get('short'):
        return gce_series_getattrs(series, 'node_id worker_id core container_name', short=True)
    return gce_series_getattrs(series, 'project_id zone node_id worker_id core container_name')


labelers = {
    "compute.googleapis.com/instance/network/sent_bytes_count":
    gce_instance_labeler,
    "compute.googleapis.com/instance/network/received_bytes_count":
    gce_instance_labeler,
    "compute.googleapis.com/instance/cpu/utilization":
    gce_instance_labeler,
    "compute.googleapis.com/instance/disk/write_bytes_count":
    gce_instance_disk_labeler,
    "compute.googleapis.com/instance/disk/read_bytes_count":
    gce_instance_disk_labeler,
    "tpu.googleapis.com/tpu/mxu/utilization":
    gce_tpu_labeler,
    "tpu.googleapis.com/container/memory/usage":
    gce_tpu_labeler,
    "tpu.googleapis.com/cpu/utilization":
    gce_tpu_labeler,
    "tpu.googleapis.com/memory/usage":
    gce_tpu_labeler,
    "tpu.googleapis.com/network/sent_bytes_count":
    gce_tpu_labeler,
    "tpu.googleapis.com/network/received_bytes_count":
    gce_tpu_labeler,
}


def get_time_series_label(ts, **options):
    return labelers[ts.metric.type](ts, **options)

def get_default_project_id():
    _, project_id = google.auth.default()
    return project_id

class TimeSeriesMonitor:
    def __init__(self, project_id=None, client=None):
        if project_id is None:
            project_id = get_default_project_id()
        elif project_id in ['tfork', 'tensorfork']:
            project_id = 'gpt-2-15b-poetry'
        self.project_id = project_id
        if client is None:
            client = monitoring_v3.MetricServiceClient()
        self.client = client

    def __call__(self, *args, **kwargs):
        return self.get(*args, **kwargs)

    def get(self, metric="tpu_mxu", node_id=None, interval=None, filters=None, raw=False, when=None, full_names=False):
        if when is None:
            when = utc()

        if '/' not in metric:
            metric = metrics[metric]

        if interval is None:
            now = time.time()
            seconds = int(now)
            nanos = int((now - seconds) * 10 ** 9)
            interval = monitoring_v3.TimeInterval(
                {
                    "end_time": {"seconds": seconds, "nanos": nanos},
                    "start_time": {"seconds": (seconds - 1200), "nanos": nanos},
                }
            )

        if filters is None:
            filters = []
        filters = filters[:]
        if node_id is not None:
            filters += [['resource.labels.node_id', node_id]]
        filters += [['metric.type', metric]]
        filters = ' AND '.join(['{} = {}'.format(k, json.dumps(v)) for k, v in filters])

        results = self.client.list_time_series(
            request={
                "name": "projects/{project_id}".format(project_id=self.project_id),
                "filter": filters,
                "interval": interval,
                "view": monitoring_v3.ListTimeSeriesRequest.TimeSeriesView.FULL,
            }
        )
        if raw:
            return results
        points = collections.defaultdict(lambda: [])
        for timeSeries in results:
            key = get_time_series_label(timeSeries, short=not full_names)
        for point in timeSeries.points:
            point_utc = point.interval.start_time.timestamp()
            seconds_ago = int(when - point_utc)
            if timeSeries.value_type == 2: # what's the correct way to get INT64 here?
                value = point.value.int64_value
            else:
                value = point.value.double_value
            points[key].append([seconds_ago, value])
        points = dict(points)
        return points


def get_workers_list(cluster_resolver):
    worker_job_name = 'worker'
    cluster_spec = cluster_resolver.cluster_spec()
    if not cluster_spec:
        raise errors.UnavailableError(
            'None', 'None',
            'Cluster spec not found, your client must run in GCE environment.')
    task_indices = cluster_spec.task_indices(worker_job_name)
    workers_list = [
        cluster_spec.task_address(worker_job_name, i).replace(':8470', ':8466')
        for i in task_indices
    ]
    return ','.join(workers_list)


def parse_tpu_data(tpu):
    data = tpu['name'].split('/')
    tpu_name, tpu_zone = data[-1], data[-3]
    tpu_config = {
        'name': tpu_name,
        'mesh': tpu['acceleratorType'],
        'region': tpu_zone,
        'master': tpu['ipAddress'] if 'ipAddress' in tpu else None,
    }
    return tpu_config

def tpunicorn_query(project):
    if project in ['tfork', 'tensorfork']:
        project = 'gpt-2-15b-poetry'
    config = {'project': project}
    if not env['colab']:
        import tpunicorn
        tpu_data = None
        for zone in ['europe-west4-a', 'us-central1-f', 'us-central1-a', 'us-central1-b', 'us-central1-c', 'asia-east1-c']:
            try:
                tpu_data = tpunicorn.tpu.get_tpus(zone=zone, project=project)
                if tpu_data:
                    break

            except:
                continue
        
        selected_tpu = None
        tpu_name = os.environ.get('TPU_NAME', None)
        if not tpu_data:
            print('Failed to find a TPU - Ensure you have the correct GOOGLE_APPLICATION_CREDENTIALS set for your project')
            sys.exit()
        if len(tpu_data) > 1:
            if tpu_name:
                for x, tpu in enumerate(tpu_data):
                    if tpu_name in tpu['name']:
                        selected_tpu = tpu_data[x]
            else:
                for x, tpu in enumerate(tpu_data):
                    print(f'[{x}] - {tpu}')
                
                tpu_idx = input('Select TPU')
                selected_tpu = tpu_data[tpu_idx]
            
        else:
            selected_tpu = tpu_data[0]

        tpu_config = parse_tpu_data(selected_tpu)
        config.update(tpu_config)
        
    else:
        config['master'] = os.environ['TPU_NAME']
        config['name'] = os.environ['TPU_NAME']
        config['region'] = 'us'
        config['mesh'] = 'v2-8'
    return config


import os, sys
import os.path

from setuptools import setup, find_packages

root = os.path.abspath(os.path.dirname(__file__))
package_name = "tpubar"
packages = find_packages(
    include=[package_name, "{}.*".format(package_name)]
)

_locals = {}
with open(os.path.join(package_name, "_version.py")) as fp:
    exec(fp.read(), None, _locals)

version = _locals["__version__"]
binary_names = _locals["binary_names"]

with open(os.path.join(root, 'README.md'), 'rb') as readme:
    long_description = readme.read().decode('utf-8')

setup(
    name=package_name,
    version=version,
    description="tpubar",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Tri Songz',
    author_email='ts@scontentenginex.com',
    keywords=['tpu', 'progress bar', 'monitoring', 'google cloud', 'tensorflow'],
    url='http://github.com/trisongz/tpubar',
    python_requires='>3.6',
    install_requires=[
        "tqdm>=4.50.0",
        "google-cloud-monitoring",
        "tensorflow",
        "psutil",
        "click",
        "pysimdjson",
        "tpunicorn",
    ],
    packages=packages,
    package_data = {
        'json': ['*.json'],
    },
    include_package_data=True,
    entry_points={
        "console_scripts": [
            "{} = {}.cli:cli".format(binary_name, package_name)
            for binary_name in binary_names
        ]
    },
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        "Intended Audience :: Developers",
    ],
)


import click
import json
import sys
import os
os.environ['TF_CPP_MIN_LOG_LEVEL'] = '0'
import time

from tpubar.utils import run_command

@click.group()
@click.pass_context
def cli(ctx, **kws):
    ctx.obj = kws

@cli.command('monitor')
@click.argument('tpu_name', type=click.STRING, default=os.environ.get('TPU_NAME', None))
@click.option('--project', type=click.STRING, default=None)
@click.option('-v', '--verbose', is_flag=True)
def monitor_tpubar(tpu_name, project, verbose):
    tpu_name = tpu_name if tpu_name else os.environ.get('TPU_NAME', None)
    if not tpu_name:
        tpu_name = click.prompt('Please enter a TPU Name', type=click.STRING)
        if not tpu_name:
            raise ValueError('Valid TPU Name must be selected')
    if not os.environ.get('GOOGLE_APPLICATION_CREDENTIALS', None):
        adc = click.prompt('Please enter a path to GOOGLE_APPLICATION_CREDENTIALS', type=click.STRING)
        if adc:
            os.environ['GOOGLE_APPLICATION_CREDENTIALS'] = adc
    
    click.echo(f'Monitoring TPU: {tpu_name} until cancelled.')
    from tpubar import TPUMonitor, env
    if env['colab']:
        monitor = TPUMonitor(tpu_name=tpu_name, project=project, profiler='v2', refresh_secs=3, verbose=verbose)
    else:
        monitor = TPUMonitor(tpu_name=tpu_name, project=project, profiler='v1', refresh_secs=3, verbose=verbose)

    monitor.start()
    while True:
        try:
            time.sleep(10)
        except KeyboardInterrupt:
            click.echo(f'\nShutting Down Monitor')
            monitor.close()
            sys.exit()

@cli.command('test')
@click.argument('tpu_name', type=click.STRING, default=os.environ.get('TPU_NAME', None))
@click.option('--project', type=click.STRING, default=None)
def test_tpubar(tpu_name, project):
    tpu_name = tpu_name if tpu_name else os.environ.get('TPU_NAME', None)
    if not tpu_name:
        tpu_name = click.prompt('Please enter a TPU Name', type=click.STRING)
        if not tpu_name:
            raise ValueError('Valid TPU Name must be selected')
    if not os.environ.get('GOOGLE_APPLICATION_CREDENTIALS', None):
        adc = click.prompt('Please enter a path to GOOGLE_APPLICATION_CREDENTIALS', type=click.STRING)
        if adc:
            os.environ['GOOGLE_APPLICATION_CREDENTIALS'] = adc
    
    click.echo(f'Running Test for TPUBar on TPU {tpu_name}')
    from tpubar import TPUMonitor, env
    if env['colab']:
        monitor = TPUMonitor(tpu_name=tpu_name, project=project, profiler='v2', refresh_secs=3, verbose=True)
    else:
        #click.echo(f'{project}')
        monitor = TPUMonitor(tpu_name=tpu_name, project=project, profiler='v1', refresh_secs=3, verbose=True)

    monitor.start()
    for x in range(6):
        time.sleep(10)
    click.echo(f'\nCompleted Testing')

@cli.command('trace')
@click.argument('tpu_name', type=click.STRING, default=os.environ.get('TPU_NAME', None))
@click.option('-v', '--verbose', is_flag=True)
def trace_tpubar(tpu_name, verbose):
    tpu_name = tpu_name if tpu_name else os.environ.get('TPU_NAME', None)
    if not tpu_name:
        tpu_name = click.prompt('Please enter a TPU Name', type=click.STRING)
        if not tpu_name:
            raise ValueError('Valid TPU Name must be selected')
    if not os.environ.get('GOOGLE_APPLICATION_CREDENTIALS', None):
        adc = click.prompt('Please enter a path to GOOGLE_APPLICATION_CREDENTIALS', type=click.STRING)
        if adc:
            os.environ['GOOGLE_APPLICATION_CREDENTIALS'] = adc
    
    click.echo(f'Tracing TPU: {tpu_name} until cancelled.')
    from tpubar import TPUMonitor, env

    monitor = TPUMonitor(tpu_name=tpu_name, profiler='trace', refresh_secs=10, verbose=verbose)
    monitor.trace()
    while True:
        try:
            time.sleep(10)
        except KeyboardInterrupt:
            click.echo(f'\nShutting Down Tracer')
            sys.exit()

@cli.command('auth')
@click.argument('auth_name', type=click.STRING, default=os.environ.get('GOOGLE_APPLICATION_CREDENTIALS', None))
@click.option('-l', '--list_auths', is_flag=True)
def set_auth(auth_name, list_auths):
    from tpubar import env, auths, update_auth
    click.echo('\n')
    if list_auths:
        click.echo('Listing Auths')
        for name, adc_path in auths.items():
            click.echo(f'- {name}: {adc_path}')
        click.echo('\n')

    click.echo(f'Current ADC is set to {os.environ.get("GOOGLE_APPLICATION_CREDENTIALS", "None")}')
    if auth_name in auths.keys():
        if auths[auth_name] not in auths.values():
            click.echo(f'Setting {auth_name} to BACKUP_ADC_PATH')
            auths['BACKUP_ADC_PATH'] = auths[auth_name]
        click.echo(f'- {auth_name} is now the Default ADC: {auths[auth_name]}')
        auths['DEFAULT_ADC'] = auths[auth_name]
    
    else:
        click.echo(f'{auth_name} was not found in {list(auths.keys())} - Creating New Auth')
        adc_name = click.prompt('Please enter a name for your ADC', type=click.STRING)
        adc_path = click.prompt('Please enter a path to GOOGLE_APPLICATION_CREDENTIALS', type=click.STRING)
        assert os.path.exists(adc_path), 'Path to GOOGLE_APPLICATION_CREDENTIALS was not found. Exiting'
        auths.update({adc_name: adc_path})
        auths['DEFAULT_ADC'] = adc_path
        click.echo(f'- {adc_name} is now the Default ADC: {adc_path}')
    update_auth(auths)
    os.environ['GOOGLE_APPLICATION_CREDENTIALS'] = auths['DEFAULT_ADC']


@cli.command('sess')
@click.argument('session_name', default='train')
def create_sess(session_name):
    _conda_exe = os.getenv('CONDA_EXE').replace('bin/conda', 'etc/profile.d/conda.sh')
    _conda_env = os.getenv('CONDA_DEFAULT_ENV', None)
    command = f'tmux new -d -s {session_name}'
    os.system(command)
    if _conda_env:
        command = f'tmux send-keys -t {session_name}.0 "source {_conda_exe} && conda deactivate && conda activate {_conda_env} && clear && cd {os.getcwd()}" ENTER'
        os.system(command)
    os.system(f'tmux a -t {session_name}')

@cli.command('attach')
@click.argument('session_name', default='train')
def attach_sess(session_name):
    command = f'tmux a -t {session_name}'
    os.system(command)

@cli.command('killsess')
@click.argument('session_name', default='train')
def kill_sess(session_name):
    click.echo(f'Killing {session_name}')
    command = f'tmux kill-session -t {session_name}'
    os.system(command)


def main(*args, prog_name='tpubar', auto_envvar_prefix='TPUBAR', **kws):
    cli.main(*args, prog_name=prog_name, auto_envvar_prefix=auto_envvar_prefix, **kws)

if __name__ == "__main__":
    main()